@@ -116,29 +116,8 @@ wnck_application_init (WnckApplication *application)
 {
   application->priv = WNCK_APPLICATION_GET_PRIVATE (application);
 
-  application->priv->xwindow = None;
-  application->priv->screen = NULL;
-  application->priv->windows = NULL;
-  application->priv->pid = 0;
-  application->priv->name = NULL;
-
-  application->priv->name_window = NULL;
-
-  application->priv->icon = NULL;
-  application->priv->mini_icon = NULL;
-
   application->priv->icon_cache = _wnck_icon_cache_new ();
-  _wnck_icon_cache_set_want_fallback (application->priv->icon_cache,
-                                      FALSE);
-
-  application->priv->icon_window = NULL;
-
-  application->priv->startup_id = NULL;
-
-  application->priv->name_from_leader = FALSE;
-  application->priv->icon_from_leader = FALSE;
-
-  application->priv->need_emit_icon_changed = FALSE;
+  _wnck_icon_cache_set_want_fallback (application->priv->icon_cache, FALSE);
 }
 
 static void
@@ -161,8 +140,7 @@ wnck_application_class_init (WnckApplicationClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckApplicationClass, name_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   /**
@@ -176,8 +154,7 @@ wnck_application_class_init (WnckApplicationClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckApplicationClass, icon_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 }
 
@@ -360,18 +337,19 @@ get_icons (WnckApplication *app)
 {
   GdkPixbuf *icon;
   GdkPixbuf *mini_icon;
+  gsize normal_size;
+  gsize mini_size;
 
   icon = NULL;
   mini_icon = NULL;
+  normal_size = _wnck_get_default_icon_size ();
+  mini_size = _wnck_get_default_mini_icon_size ();
 
   if (_wnck_read_icons (WNCK_SCREEN_XSCREEN (app->priv->screen),
                         app->priv->xwindow,
                         app->priv->icon_cache,
-                        &icon,
-                        DEFAULT_ICON_WIDTH, DEFAULT_ICON_HEIGHT,
-                        &mini_icon,
-                        DEFAULT_MINI_ICON_WIDTH,
-                        DEFAULT_MINI_ICON_HEIGHT))
+                        &icon, normal_size, normal_size,
+                        &mini_icon, mini_size, mini_size))
     {
       app->priv->need_emit_icon_changed = TRUE;
       app->priv->icon_from_leader = TRUE;
@@ -396,6 +374,16 @@ get_icons (WnckApplication *app)
             !(app->priv->icon || app->priv->mini_icon));
 }
 
+void
+_wnck_application_load_icons (WnckApplication *app)
+{
+  g_return_if_fail (WNCK_IS_APPLICATION (app));
+
+  get_icons (app);
+  if (app->priv->need_emit_icon_changed)
+    emit_icon_changed (app);
+}
+
 /* Prefer to get group icon from a window of type "normal" */
 static WnckWindow*
 find_icon_window (WnckApplication *app)
@@ -436,9 +424,7 @@ wnck_application_get_icon (WnckApplication *app)
 {
   g_return_val_if_fail (WNCK_IS_APPLICATION (app), NULL);
 
-  get_icons (app);
-  if (app->priv->need_emit_icon_changed)
-    emit_icon_changed (app);
+  _wnck_application_load_icons (app);
 
   if (app->priv->icon)
     return app->priv->icon;
@@ -469,9 +455,7 @@ wnck_application_get_mini_icon (WnckApplication *app)
 {
   g_return_val_if_fail (WNCK_IS_APPLICATION (app), NULL);
 
-  get_icons (app);
-  if (app->priv->need_emit_icon_changed)
-    emit_icon_changed (app);
+  _wnck_application_load_icons (app);
 
   if (app->priv->mini_icon)
     return app->priv->mini_icon;
