@@ -22,6 +22,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#undef WNCK_DISABLE_DEPRECATED
+
 #include <string.h>
 #include "class-group.h"
 #include "window.h"
@@ -78,6 +80,16 @@ enum {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
+void
+_wnck_class_group_shutdown_all (void)
+{
+  if (class_group_hash != NULL)
+    {
+      g_hash_table_destroy (class_group_hash);
+      class_group_hash = NULL;
+    }
+}
+
 static void
 wnck_class_group_class_init (WnckClassGroupClass *class)
 {
@@ -98,8 +110,7 @@ wnck_class_group_class_init (WnckClassGroupClass *class)
                   G_OBJECT_CLASS_TYPE (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckClassGroupClass, name_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
   /**
    * WnckClassGroup::icon-changed:
@@ -112,8 +123,7 @@ wnck_class_group_class_init (WnckClassGroupClass *class)
                   G_OBJECT_CLASS_TYPE (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckClassGroupClass, icon_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 }
 
@@ -121,13 +131,6 @@ static void
 wnck_class_group_init (WnckClassGroup *class_group)
 {
   class_group->priv = WNCK_CLASS_GROUP_GET_PRIVATE (class_group);
-
-  class_group->priv->res_class = NULL;
-  class_group->priv->name = NULL;
-  class_group->priv->windows = NULL;
-
-  class_group->priv->icon = NULL;
-  class_group->priv->mini_icon = NULL;
 }
 
 static void
@@ -161,24 +164,24 @@ wnck_class_group_finalize (GObject *object)
 
 /**
  * wnck_class_group_get:
- * @res_class: name of the sought resource class.
+ * @id: identifier name of the sought resource class.
  *
- * Gets the #WnckClassGroup corresponding to @res_class.
+ * Gets the #WnckClassGroup corresponding to @id.
  *
  * Return value: (transfer none): the #WnckClassGroup corresponding to
- * @res_class, or %NULL if there is no #WnckClassGroup with the specified
- * @res_class. The returned #WnckClassGroup is owned by libwnck and must not be
+ * @id, or %NULL if there is no #WnckClassGroup with the specified
+ * @id. The returned #WnckClassGroup is owned by libwnck and must not be
  * referenced or unreferenced.
  *
  * Since: 2.2
  **/
 WnckClassGroup *
-wnck_class_group_get (const char *res_class)
+wnck_class_group_get (const char *id)
 {
   if (!class_group_hash)
     return NULL;
   else
-    return g_hash_table_lookup (class_group_hash, res_class ? res_class : "");
+    return g_hash_table_lookup (class_group_hash, id ? id : "");
 }
 
 /**
@@ -198,7 +201,8 @@ _wnck_class_group_create (const char *res_class)
   WnckClassGroup *class_group;
 
   if (class_group_hash == NULL)
-    class_group_hash = g_hash_table_new (g_str_hash, g_str_equal);
+    class_group_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                              NULL, g_object_unref);
 
   g_return_val_if_fail (g_hash_table_lookup (class_group_hash, res_class ? res_class : "") == NULL,
 			NULL);
@@ -227,11 +231,7 @@ _wnck_class_group_destroy (WnckClassGroup *class_group)
 
   g_hash_table_remove (class_group_hash, class_group->priv->res_class);
 
-  g_free (class_group->priv->res_class);
-  class_group->priv->res_class = NULL;
-
-  /* remove hash's ref on the class group */
-  g_object_unref (class_group);
+  /* Removing from hash also removes the only ref WnckClassGroup had */
 }
 
 static const char *
@@ -413,6 +413,7 @@ static void
 set_icon (WnckClassGroup *class_group)
 {
   GdkPixbuf *icon, *mini_icon;
+  gboolean icons_reffed = FALSE;
 
   get_icons_from_applications (class_group, &icon, &mini_icon);
 
@@ -420,12 +421,15 @@ set_icon (WnckClassGroup *class_group)
     get_icons_from_windows (class_group, &icon, &mini_icon);
 
   if (!icon || !mini_icon)
+    {
       _wnck_get_fallback_icons (&icon,
-                                DEFAULT_ICON_WIDTH,
-                                DEFAULT_ICON_HEIGHT,
+                                _wnck_get_default_icon_size (),
+                                _wnck_get_default_icon_size (),
                                 &mini_icon,
-                                DEFAULT_MINI_ICON_WIDTH,
-                                DEFAULT_MINI_ICON_HEIGHT);
+                                _wnck_get_default_mini_icon_size (),
+                                _wnck_get_default_mini_icon_size ());
+      icons_reffed = TRUE;
+    }
 
   g_assert (icon && mini_icon);
 
@@ -435,8 +439,14 @@ set_icon (WnckClassGroup *class_group)
   if (class_group->priv->mini_icon)
     g_object_unref (class_group->priv->mini_icon);
 
-  class_group->priv->icon = g_object_ref (icon);
-  class_group->priv->mini_icon = g_object_ref (mini_icon);
+  class_group->priv->icon = icon;
+  class_group->priv->mini_icon = mini_icon;
+
+  if (!icons_reffed)
+    {
+      g_object_ref (class_group->priv->icon);
+      g_object_ref (class_group->priv->mini_icon);
+    }
 
   g_signal_emit (G_OBJECT (class_group), signals[ICON_CHANGED], 0);
 }
@@ -474,7 +484,7 @@ _wnck_class_group_add_window (WnckClassGroup *class_group,
  * _wnck_class_group_remove_window:
  * @class_group: a #WnckClassGroup.
  * @window: a #WnckWindow.
- * 
+ *
  * Removes a window from the list of windows that are grouped under the
  * specified @class_group.
  **/
@@ -497,9 +507,9 @@ _wnck_class_group_remove_window (WnckClassGroup *class_group,
 /**
  * wnck_class_group_get_windows:
  * @class_group: a #WnckClassGroup.
- * 
+ *
  * Gets the list of #WnckWindow that are grouped in @class_group.
- * 
+ *
  * Return value: (element-type WnckWindow) (transfer none): the list of
  * #WnckWindow grouped in @class_group, or %NULL if the group contains no
  * window. The list should not be modified nor freed, as it is owned by
@@ -515,16 +525,37 @@ wnck_class_group_get_windows (WnckClassGroup *class_group)
   return class_group->priv->windows;
 }
 
+/**
+ * wnck_class_group_get_id:
+ * @class_group: a #WnckClassGroup.
+ *
+ * Gets the identifier name for @class_group. This is the resource class for
+ * @class_group.
+ *
+ * Return value: the identifier name of @class_group, or an
+ * empty string if the group has no identifier name.
+ *
+ * Since: 3.2
+ **/
+const char *
+wnck_class_group_get_id (WnckClassGroup *class_group)
+{
+  g_return_val_if_fail (class_group != NULL, NULL);
+
+  return class_group->priv->res_class;
+}
+
 /**
  * wnck_class_group_get_res_class:
  * @class_group: a #WnckClassGroup.
- * 
+ *
  * Gets the resource class name for @class_group.
- * 
+ *
  * Return value: the resource class name of @class_group, or an
  * empty string if the group has no resource class name.
  *
  * Since: 2.2
+ * Deprecated:3.2: Use wnck_class_group_get_id() instead.
  **/
 const char *
 wnck_class_group_get_res_class (WnckClassGroup *class_group)
@@ -537,7 +568,7 @@ wnck_class_group_get_res_class (WnckClassGroup *class_group)
 /**
  * wnck_class_group_get_name:
  * @class_group: a #WnckClassGroup.
- * 
+ *
  * Gets an human-readable name for @class_group. Since there is no way to
  * properly find this name, a suboptimal heuristic is used to find it. The name
  * is the name of all #WnckApplication for each #WnckWindow in @class_group if
@@ -545,7 +576,7 @@ wnck_class_group_get_res_class (WnckClassGroup *class_group)
  * name, the name is the name of all #WnckWindow in @class_group if they all
  * have the same name. If all #WnckWindow don't have the same name, the
  * resource class name is used.
- * 
+ *
  * Return value: an human-readable name for @class_group.
  *
  * Since: 2.2
@@ -561,16 +592,16 @@ wnck_class_group_get_name (WnckClassGroup *class_group)
 /**
  * wnck_class_group_get_icon:
  * @class_group: a #WnckClassGroup.
- * 
+ *
  * Gets the icon to be used for @class_group. Since there is no way to
  * properly find the icon, a suboptimal heuristic is used to find it. The icon
  * is the first icon found by looking at all the #WnckApplication for each
  * #WnckWindow in @class_group, then at all the #WnckWindow in @class_group. If
  * no icon was found, a fallback icon is used.
- * 
- * Return value: the icon for @class_group. The caller should reference the
- * returned <classname>GdkPixbuf</classname> if it needs to keep the icon
- * around.
+ *
+ * Return value: (transfer none): the icon for @class_group. The caller should
+ * reference the returned <classname>GdkPixbuf</classname> if it needs to keep
+ * the icon around.
  *
  * Since: 2.2
  **/
@@ -585,14 +616,14 @@ wnck_class_group_get_icon (WnckClassGroup *class_group)
 /**
  * wnck_class_group_get_mini_icon:
  * @class_group: a #WnckClassGroup.
- * 
+ *
  * Gets the mini-icon to be used for @class_group. Since there is no way to
  * properly find the mini-icon, the same suboptimal heuristic as the one for
  * wnck_class_group_get_icon() is used to find it.
- * 
- * Return value: the mini-icon for @class_group. The caller should reference
- * the returned <classname>GdkPixbuf</classname> if it needs to keep the
- * mini-icon around.
+ *
+ * Return value: (transfer none): the mini-icon for @class_group. The caller
+ * should reference the returned <classname>GdkPixbuf</classname> if it needs
+ * to keep the mini-icon around.
  *
  * Since: 2.2
  **/
