@@ -203,47 +203,9 @@ wnck_screen_init (WnckScreen *screen)
   screen->priv = WNCK_SCREEN_GET_PRIVATE (screen);
 
   screen->priv->number = -1;
-  screen->priv->xroot = None;
-  screen->priv->xscreen = NULL;
-
-  screen->priv->mapped_windows = NULL;
-  screen->priv->stacked_windows = NULL;
-  screen->priv->workspaces = NULL;
-
-  screen->priv->active_window = NULL;
-  screen->priv->previously_active_window = NULL;
-
-  screen->priv->active_workspace = NULL;
-
-  screen->priv->window_order = 0;
-
-  screen->priv->bg_pixmap = None;
-
-  screen->priv->wm_name = NULL;
-
-  screen->priv->update_handler = 0;
-
-#ifdef HAVE_STARTUP_NOTIFICATION
-  screen->priv->sn_display = NULL;
-#endif
-
-  screen->priv->showing_desktop = FALSE;
-
-  screen->priv->vertical_workspaces = FALSE;
   screen->priv->starting_corner = WNCK_LAYOUT_CORNER_TOPLEFT;
   screen->priv->rows_of_workspaces = 1;
   screen->priv->columns_of_workspaces = -1;
-
-  screen->priv->need_update_stack_list = FALSE;
-  screen->priv->need_update_workspace_list = FALSE;
-  screen->priv->need_update_viewport_settings = FALSE;
-  screen->priv->need_update_active_workspace = FALSE;
-  screen->priv->need_update_active_window = FALSE;
-  screen->priv->need_update_workspace_layout = FALSE;
-  screen->priv->need_update_workspace_names = FALSE;
-  screen->priv->need_update_bg_pixmap = FALSE;
-  screen->priv->need_update_showing_desktop = FALSE;
-  screen->priv->need_update_wm = FALSE;
 }
 
 static void
@@ -270,8 +232,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, active_window_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__OBJECT,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1, WNCK_TYPE_WINDOW);
 
   /**
@@ -287,8 +248,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, active_workspace_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__OBJECT,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1, WNCK_TYPE_WORKSPACE);
 
   /**
@@ -302,8 +262,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, window_stacking_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   /**
@@ -318,8 +277,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, window_opened),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__OBJECT,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1, WNCK_TYPE_WINDOW);
 
   /**
@@ -334,8 +292,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, window_closed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__OBJECT,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1, WNCK_TYPE_WINDOW);
 
   /**
@@ -350,8 +307,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, workspace_created),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__OBJECT,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1, WNCK_TYPE_WORKSPACE);
 
   /**
@@ -366,8 +322,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, workspace_destroyed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__OBJECT,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1, WNCK_TYPE_WORKSPACE);
 
   /**
@@ -382,8 +337,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, application_opened),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__OBJECT,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1, WNCK_TYPE_APPLICATION);
 
   /**
@@ -398,8 +352,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, application_closed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__OBJECT,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1, WNCK_TYPE_APPLICATION);
 
   /**
@@ -416,8 +369,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, class_group_opened),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__OBJECT,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1, WNCK_TYPE_CLASS_GROUP);
 
   /**
@@ -434,8 +386,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, class_group_closed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__OBJECT,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1, WNCK_TYPE_CLASS_GROUP);
 
   /**
@@ -449,8 +400,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, background_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   /**
@@ -466,8 +416,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, showing_desktop_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   /**
@@ -484,8 +433,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, viewports_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   /**
@@ -501,8 +449,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckScreenClass, window_manager_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 }
 
