@@ -45,6 +45,10 @@ static void window_icon_changed_callback      (WnckWindow      *window,
                                                gpointer         data);
 static void window_geometry_changed_callback  (WnckWindow      *window,
                                                gpointer         data);
+static void window_class_changed_callback     (WnckWindow      *window,
+                                               gpointer         data);
+static void window_role_changed_callback      (WnckWindow      *window,
+                                               gpointer         data);
 
 static GtkTreeModel* create_tree_model (void);
 static GtkWidget*    create_tree_view  (void);
@@ -54,13 +58,30 @@ static void          update_window     (GtkTreeModel *model,
                                         WnckWindow   *window);
 static void          queue_refill_model (void);
 
+static gint icon_size = WNCK_DEFAULT_MINI_ICON_SIZE;
+
+static GOptionEntry entries[] = {
+  {"icon-size", 'i', 0, G_OPTION_ARG_INT, &icon_size, "Icon size for tasklist", NULL},
+  {NULL }
+};
+
+
 int
 main (int argc, char **argv)
 {
   WnckScreen *screen;
   GtkWidget *sw;
   GtkWidget *win;
-  
+  GOptionContext *ctxt;
+
+  ctxt = g_option_context_new ("");
+  g_option_context_add_main_entries (ctxt, entries, NULL);
+  g_option_context_add_group (ctxt, gtk_get_option_group (TRUE));
+  g_option_context_parse (ctxt, &argc, &argv, NULL);
+  g_clear_pointer (&ctxt, g_option_context_free);
+
+  wnck_set_default_mini_icon_size (icon_size);
+
   gtk_init (&argc, &argv);
 
   screen = wnck_screen_get (0);
@@ -163,12 +184,14 @@ window_opened_callback            (WnckScreen    *screen,
                                    WnckWindow    *window,
                                    gpointer       data)
 {
-  g_print ("Window '%s' opened (pid = %d session_id = %s)\n",
+  g_print ("Window '%s' opened (pid = %d, session_id = %s, role = %s)\n",
            wnck_window_get_name (window),
            wnck_window_get_pid (window),
            wnck_window_get_session_id (window) ?
-           wnck_window_get_session_id (window) : "none");
-  
+            wnck_window_get_session_id (window) : "none",
+           wnck_window_get_role (window) ?
+            wnck_window_get_role (window) : "none");
+
   g_signal_connect (G_OBJECT (window), "name_changed",
                     G_CALLBACK (window_name_changed_callback),
                     NULL);
@@ -177,14 +200,20 @@ window_opened_callback            (WnckScreen    *screen,
                     NULL);
   g_signal_connect (G_OBJECT (window), "workspace_changed",
                     G_CALLBACK (window_workspace_changed_callback),
-                    NULL);    
+                    NULL);
   g_signal_connect (G_OBJECT (window), "icon_changed",
                     G_CALLBACK (window_icon_changed_callback),
                     NULL);
   g_signal_connect (G_OBJECT (window), "geometry_changed",
                     G_CALLBACK (window_geometry_changed_callback),
                     NULL);
-  
+  g_signal_connect (G_OBJECT (window), "class_changed",
+                    G_CALLBACK (window_class_changed_callback),
+                    NULL);
+  g_signal_connect (G_OBJECT (window), "role_changed",
+                    G_CALLBACK (window_role_changed_callback),
+                    NULL);
+
   queue_refill_model ();
 }
 
@@ -336,11 +365,37 @@ window_geometry_changed_callback  (WnckWindow      *window,
   int x, y, width, height;
 
   wnck_window_get_geometry (window, &x, &y, &width, &height);
-  
+
   g_print ("Geometry changed on window '%s': %d,%d  %d x %d\n",
            wnck_window_get_name (window), x, y, width, height);
 }
 
+static void
+window_class_changed_callback  (WnckWindow      *window,
+                                gpointer         data)
+{
+  const char *group_name;
+  const char *instance_name;
+
+  group_name = wnck_window_get_class_group_name (window);
+  instance_name = wnck_window_get_class_instance_name (window);
+
+  g_print ("Class changed on window '%s': %s,%s\n",
+           wnck_window_get_name (window), group_name, instance_name);
+}
+
+static void
+window_role_changed_callback  (WnckWindow      *window,
+                               gpointer         data)
+{
+  const char *role;
+
+  role = wnck_window_get_role (window);
+
+  g_print ("Role changed on window '%s': %s\n",
+           wnck_window_get_name (window), role);
+}
+
 static GtkTreeModel*
 create_tree_model (void)
 {
