@@ -54,6 +54,15 @@
  * libwnck users.
  */
 
+/**
+ * SECTION:icons
+ * @short_description: icons related functions.
+ * @stability: Unstable
+ *
+ * These functions are utility functions to manage icons for #WnckWindow and
+ * #WnckApplication.
+ */
+
 typedef enum
 {
   WNCK_EXT_UNKNOWN = 0,
@@ -657,11 +666,79 @@ _wnck_get_client_type (void)
   return client_type;
 }
 
+static gsize default_icon_size = WNCK_DEFAULT_ICON_SIZE;
+
+/**
+ * wnck_set_default_icon_size:
+ * @size: the default size for windows and application standard icons.
+ *
+ * The default main icon size is %WNCK_DEFAULT_ICON_SIZE. This function allows
+ * to change this value.
+ *
+ * Since: 2.4.6
+ */
+void
+wnck_set_default_icon_size (gsize size)
+{
+  default_icon_size = size;
+}
+
+gsize
+_wnck_get_default_icon_size (void)
+{
+  return default_icon_size;
+}
+
+static gsize default_mini_icon_size = WNCK_DEFAULT_MINI_ICON_SIZE;
+
+/**
+ * wnck_set_default_mini_icon_size:
+ * @size: the default size for windows and application mini icons.
+ *
+ * The default main icon size is %WNCK_DEFAULT_MINI_ICON_SIZE. This function
+ * allows to change this value.
+ *
+ * Since: 2.4.6
+ */
+void
+wnck_set_default_mini_icon_size (gsize size)
+{
+  int default_screen;
+  WnckScreen *screen;
+  GList *l;
+
+  default_mini_icon_size = size;
+
+  default_screen = DefaultScreen (_wnck_get_default_display ());
+  screen = _wnck_screen_get_existing (default_screen);
+
+  if (WNCK_IS_SCREEN (screen))
+    {
+      /* Make applications and icons to reload their icons */
+      for (l = wnck_screen_get_windows (screen); l; l = l->next)
+        {
+          WnckWindow *window = WNCK_WINDOW (l->data);
+          WnckApplication *application = wnck_window_get_application (window);
+
+          _wnck_window_load_icons (window);
+
+          if (WNCK_IS_APPLICATION (application))
+            _wnck_application_load_icons (application);
+        }
+    }
+}
+
+gsize
+_wnck_get_default_mini_icon_size (void)
+{
+  return default_mini_icon_size;
+}
+
 /**
  * _make_gtk_label_bold:
+ * @label: The label.
  *
  * Switches the font of label to a bold equivalent.
- * @label: The label.
  **/
 void
 _make_gtk_label_bold (GtkLabel *label)
