@@ -21,8 +21,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#undef WNCK_DISABLE_DEPRECATED
-
 #include <config.h>
 
 #include <string.h>
@@ -318,7 +316,6 @@ item_activated_callback (GtkWidget *menu_item,
         {
           WnckWorkspace *workspace;
           int new_viewport_x, new_viewport_y;
-          int width, height;
           int xw, yw, ww, hw;
           int viewport_x, viewport_y;
 
@@ -331,9 +328,6 @@ item_activated_callback (GtkWidget *menu_item,
 
           workspace = wnck_screen_get_workspace (screen, 0);
 
-          width = wnck_screen_get_width (screen);
-          height = wnck_screen_get_height (screen);
-
           wnck_window_get_geometry (window, &xw, &yw, &ww, &hw);
 
           viewport_x = wnck_workspace_get_viewport_x (workspace);
@@ -830,7 +824,7 @@ refill_submenu_workspace (WnckActionMenu *menu)
     {
       char      *name;
       GtkWidget *item;
-	
+
       name = get_workspace_name_with_accel (menu->priv->window, i);
 
       item = make_menu_item (MOVE_TO_WORKSPACE);
@@ -1225,20 +1219,3 @@ wnck_action_menu_new (WnckWindow *window)
                        "window", window,
                        NULL);
 }
-
-/**
- * wnck_create_window_action_menu:
- * @window: the #WnckWindow for which a menu will be created.
- *
- * Creates a new #WnckActionMenu. The #WnckActionMenu will be filled with menu
- * items for window operations on @window.
- *
- * Return value: a newly created #WnckActionMenu.
- *
- * Deprecated:2.22: Use wnck_action_menu_new() instead.
- */
-GtkWidget*
-wnck_create_window_action_menu (WnckWindow *window)
-{
-	return wnck_action_menu_new (window);
-}
