@@ -35,7 +35,6 @@
 #include "xutils.h"
 #include "private.h"
 #include "wnck-enum-types.h"
-#include "wnck-marshal.h"
 
 /**
  * SECTION:window
@@ -84,6 +83,7 @@ struct _WnckWindowPrivate
   char *icon_name;
   char *session_id;
   char *session_id_utf8;
+  char *role;
   int pid;
   int workspace;
   gint sort_order;
@@ -159,9 +159,12 @@ struct _WnckWindowPrivate
   guint need_update_wmclass : 1;
   guint need_update_wmhints : 1;
   guint need_update_frame_extents : 1;
+  guint need_update_role : 1;
 
   guint need_emit_name_changed : 1;
   guint need_emit_icon_changed : 1;
+  guint need_emit_class_changed : 1;
+  guint need_emit_role_changed : 1;
 };
 
 G_DEFINE_TYPE (WnckWindow, wnck_window, G_TYPE_OBJECT);
@@ -174,6 +177,8 @@ enum {
   ICON_CHANGED,
   ACTIONS_CHANGED,
   GEOMETRY_CHANGED,
+  CLASS_CHANGED,
+  ROLE_CHANGED,
   LAST_SIGNAL
 };
 
@@ -191,6 +196,8 @@ static void emit_actions_changed   (WnckWindow       *window,
                                     WnckWindowActions changed_mask,
                                     WnckWindowActions new_actions);
 static void emit_geometry_changed  (WnckWindow      *window);
+static void emit_class_changed     (WnckWindow      *window);
+static void emit_role_changed      (WnckWindow      *window);
 
 static void update_name      (WnckWindow *window);
 static void update_state     (WnckWindow *window);
@@ -201,8 +208,9 @@ static void update_actions   (WnckWindow *window);
 static void update_wintype   (WnckWindow *window);
 static void update_transient_for (WnckWindow *window);
 static void update_startup_id (WnckWindow *window);
-static void update_wmclass    (WnckWindow *window);
+static void update_wmclass   (WnckWindow *window);
 static void update_frame_extents (WnckWindow *window);
+static void update_role      (WnckWindow *window);
 static void unqueue_update   (WnckWindow *window);
 static void queue_update     (WnckWindow *window);
 static void force_update_now (WnckWindow *window);
@@ -227,85 +235,13 @@ wnck_window_init (WnckWindow *window)
 {
   window->priv = WNCK_WINDOW_GET_PRIVATE (window);
 
-  window->priv->xwindow = None;
-  window->priv->name = NULL;
-  window->priv->app = NULL;
-  window->priv->class_group = NULL;
-  window->priv->group_leader = None;
-  window->priv->transient_for = None;
-  window->priv->orig_event_mask = 0;
+  window->priv->icon_cache = _wnck_icon_cache_new ();
   window->priv->icon_geometry.width = -1; /* invalid cached value */
-  window->priv->name = NULL;
-  window->priv->icon_name = NULL;
-  window->priv->session_id = NULL;
-  window->priv->session_id_utf8 = NULL;
-  window->priv->pid = 0;
   window->priv->workspace = -1;
   window->priv->sort_order = G_MAXINT;
 
   /* FIXME: should we have an invalid window type for this? */
-  window->priv->wintype = 0;
-
-  window->priv->icon = NULL;
-  window->priv->mini_icon = NULL;
-
-  window->priv->icon_cache = _wnck_icon_cache_new ();
-
-  window->priv->actions = 0;
-
-  window->priv->x = 0;
-  window->priv->y = 0;
-  window->priv->width = 0;
-  window->priv->height = 0;
-
-  window->priv->left_frame = 0;
-  window->priv->right_frame = 0;
-  window->priv->top_frame = 0;
-  window->priv->bottom_frame = 0;
-
-  window->priv->startup_id = NULL;
-
-  window->priv->res_class = NULL;
-  window->priv->res_name = NULL;
-
-  window->priv->transient_for_root = FALSE;
-
-  window->priv->is_minimized = FALSE;
-  window->priv->is_maximized_horz = FALSE;
-  window->priv->is_maximized_vert = FALSE;
-  window->priv->is_shaded = FALSE;
-  window->priv->is_above = FALSE;
-  window->priv->is_below = FALSE;
-  window->priv->skip_pager = FALSE;
-  window->priv->skip_taskbar = FALSE;
-  window->priv->is_sticky = FALSE;
-  window->priv->is_hidden = FALSE;
-  window->priv->is_fullscreen = FALSE;
-  window->priv->demands_attention = FALSE;
-  window->priv->is_urgent = FALSE;
-
-  window->priv->needs_attention_time = 0;
-
-  window->priv->net_wm_state_hidden = FALSE;
-  window->priv->wm_state_iconic = FALSE;
-
-  window->priv->update_handler = 0;
-
-  window->priv->need_update_name = FALSE;
-  window->priv->need_update_state = FALSE;
-  window->priv->need_update_wm_state = FALSE;
-  window->priv->need_update_icon_name = FALSE;
-  window->priv->need_update_workspace = FALSE;
-  window->priv->need_update_actions = FALSE;
-  window->priv->need_update_wintype = FALSE;
-  window->priv->need_update_transient_for = FALSE;
-  window->priv->need_update_startup_id = FALSE;
-  window->priv->need_update_wmclass = FALSE;
-  window->priv->need_update_wmhints = FALSE;
-  window->priv->need_update_frame_extents = FALSE;
-
-  window->priv->need_emit_name_changed = FALSE;
-  window->priv->need_emit_icon_changed = FALSE;
+  window->priv->wintype = WNCK_WINDOW_NORMAL;
 }
 
 static void
@@ -328,8 +264,7 @@ wnck_window_class_init (WnckWindowClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckWindowClass, name_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   /**
@@ -349,8 +284,7 @@ wnck_window_class_init (WnckWindowClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckWindowClass, state_changed),
-                  NULL, NULL,
-                  _wnck_marshal_VOID__FLAGS_FLAGS,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 2,
                   WNCK_TYPE_WINDOW_STATE, WNCK_TYPE_WINDOW_STATE);
 
@@ -366,8 +300,7 @@ wnck_window_class_init (WnckWindowClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckWindowClass, workspace_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   /**
@@ -381,8 +314,7 @@ wnck_window_class_init (WnckWindowClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckWindowClass, icon_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   /**
@@ -399,8 +331,7 @@ wnck_window_class_init (WnckWindowClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckWindowClass, actions_changed),
-                  NULL, NULL,
-                  _wnck_marshal_VOID__FLAGS_FLAGS,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 2,
                   WNCK_TYPE_WINDOW_ACTIONS,
                   WNCK_TYPE_WINDOW_ACTIONS);
@@ -416,8 +347,35 @@ wnck_window_class_init (WnckWindowClass *klass)
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckWindowClass, geometry_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 0);
+
+  /**
+   * WnckWindow::class-changed:
+   * @window: the #WnckWindow which emitted the signal.
+   *
+   * Emitted when the class name or instance name of @window changes.
+   */
+  signals[CLASS_CHANGED] =
+    g_signal_new ("class_changed",
+                  G_OBJECT_CLASS_TYPE (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (WnckWindowClass, class_changed),
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 0);
+
+  /**
+   * WnckWindow::role-changed:
+   * @window: the #WnckWindow which emitted the signal.
+   *
+   * Emitted when the role of @window changes.
+   */
+  signals[ROLE_CHANGED] =
+    g_signal_new ("role_changed",
+                  G_OBJECT_CLASS_TYPE (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (WnckWindowClass, role_changed),
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 }
 
@@ -582,8 +540,11 @@ _wnck_window_create (Window      xwindow,
   window->priv->need_update_wmclass = TRUE;
   window->priv->need_update_wmhints = TRUE;
   window->priv->need_update_frame_extents = TRUE;
+  window->priv->need_update_role = TRUE;
   window->priv->need_emit_name_changed = FALSE;
   window->priv->need_emit_icon_changed = FALSE;
+  window->priv->need_emit_class_changed = FALSE;
+  window->priv->need_emit_role_changed = FALSE;
   force_update_now (window);
 
   return window;
@@ -895,6 +856,24 @@ wnck_window_get_session_id_utf8 (WnckWindow *window)
   return window->priv->session_id_utf8;
 }
 
+/**
+ * wnck_window_get_role:
+ * @window: a #WnckWindow.
+ *
+ * Gets the role for @window.
+ * The role uniquely identifies a window among all windows that have the same
+ * client leader window.
+ *
+ * Return value: role for @window, or %NULL if @window has no role.
+ **/
+const char*
+wnck_window_get_role (WnckWindow *window)
+{
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
+
+  return window->priv->role;
+}
+
 /**
  * wnck_window_get_pid:
  * @window: a #WnckWindow.
@@ -2121,18 +2100,19 @@ get_icons (WnckWindow *window)
 {
   GdkPixbuf *icon;
   GdkPixbuf *mini_icon;
+  gsize normal_size;
+  gsize mini_size;
 
   icon = NULL;
   mini_icon = NULL;
+  normal_size = _wnck_get_default_icon_size ();
+  mini_size = _wnck_get_default_mini_icon_size ();
 
   if (_wnck_read_icons (WNCK_SCREEN_XSCREEN (window->priv->screen),
                         window->priv->xwindow,
                         window->priv->icon_cache,
-                        &icon,
-                        DEFAULT_ICON_WIDTH, DEFAULT_ICON_HEIGHT,
-                        &mini_icon,
-                        DEFAULT_MINI_ICON_WIDTH,
-                        DEFAULT_MINI_ICON_HEIGHT))
+                        &icon, normal_size, normal_size,
+                        &mini_icon, mini_size, mini_size))
     {
       window->priv->need_emit_icon_changed = TRUE;
 
@@ -2150,6 +2130,18 @@ get_icons (WnckWindow *window)
             !(window->priv->icon || window->priv->mini_icon));
 }
 
+void
+_wnck_window_load_icons (WnckWindow *window)
+{
+  g_return_if_fail (WNCK_IS_WINDOW (window));
+
+  get_icons (window);
+  if (window->priv->need_emit_icon_changed)
+    queue_update (window); /* not done in get_icons since we call that from
+                            * the update
+                            */
+}
+
 /**
  * wnck_window_get_icon:
  * @window: a #WnckWindow.
@@ -2167,11 +2159,7 @@ wnck_window_get_icon (WnckWindow *window)
 {
   g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
 
-  get_icons (window);
-  if (window->priv->need_emit_icon_changed)
-    queue_update (window); /* not done in get_icons since we call that from
-                            * the update
-                            */
+  _wnck_window_load_icons (window);
 
   return window->priv->icon;
 }
@@ -2193,11 +2181,7 @@ wnck_window_get_mini_icon (WnckWindow *window)
 {
   g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
 
-  get_icons (window);
-  if (window->priv->need_emit_icon_changed)
-    queue_update (window); /* not done in get_icons since we call that from
-                            * the update
-                            */
+  _wnck_window_load_icons (window);
 
   return window->priv->mini_icon;
 }
@@ -2624,6 +2608,12 @@ _wnck_window_process_property_notify (WnckWindow *window,
       window->priv->need_update_frame_extents = TRUE;
       queue_update (window);
     }
+  else if (xevent->xproperty.atom ==
+           _wnck_atom_get ("WM_WINDOW_ROLE"))
+    {
+      window->priv->need_update_role = TRUE;
+      queue_update (window);
+    }
 }
 
 void
@@ -3110,21 +3100,35 @@ update_startup_id (WnckWindow *window)
 static void
 update_wmclass (WnckWindow *window)
 {
+  char *new_res_class = NULL;
+  char *new_res_name = NULL;
+
   if (!window->priv->need_update_wmclass)
     return;
 
   window->priv->need_update_wmclass = FALSE;
 
-  g_free (window->priv->res_class);
-  g_free (window->priv->res_name);
-
-  window->priv->res_class = NULL;
-  window->priv->res_name = NULL;
-
   _wnck_get_wmclass (WNCK_SCREEN_XSCREEN (window->priv->screen),
                      window->priv->xwindow,
-                     &window->priv->res_class,
-                     &window->priv->res_name);
+                     &new_res_class,
+                     &new_res_name);
+
+  if (g_strcmp0 (window->priv->res_class, new_res_class) != 0 ||
+      g_strcmp0 (window->priv->res_name, new_res_name) != 0)
+    {
+      window->priv->need_emit_class_changed = TRUE;
+
+      g_free (window->priv->res_class);
+      g_free (window->priv->res_name);
+
+      window->priv->res_class = new_res_class;
+      window->priv->res_name = new_res_name;
+    }
+  else
+    {
+      g_free (new_res_class);
+      g_free (new_res_name);
+    }
 }
 
 static void
@@ -3201,6 +3205,33 @@ update_frame_extents (WnckWindow *window)
     }
 }
 
+static void
+update_role (WnckWindow *window)
+{
+  char *new_role;
+
+  if (!window->priv->need_update_role)
+    return;
+
+  window->priv->need_update_role = FALSE;
+
+  new_role = _wnck_get_text_property (WNCK_SCREEN_XSCREEN (window->priv->screen),
+                                      window->priv->xwindow,
+                                      _wnck_atom_get ("WM_WINDOW_ROLE"));
+
+  if (g_strcmp0 (window->priv->role, new_role) != 0)
+    {
+      window->priv->need_emit_role_changed = TRUE;
+
+      g_free (window->priv->role);
+      window->priv->role = new_role;
+    }
+  else
+    {
+      g_free (new_role);
+    }
+}
+
 static void
 force_update_now (WnckWindow *window)
 {
@@ -3237,6 +3268,7 @@ force_update_now (WnckWindow *window)
   update_workspace (window); /* emits signals */
   update_actions (window);
   update_frame_extents (window); /* emits signals */
+  update_role (window); /* emits signals */
 
   get_icons (window);
 
@@ -3251,6 +3283,12 @@ force_update_now (WnckWindow *window)
 
   if (window->priv->need_emit_icon_changed)
     emit_icon_changed (window);
+
+  if (window->priv->need_emit_class_changed)
+    emit_class_changed (window);
+
+  if (window->priv->need_emit_role_changed)
+    emit_role_changed (window);
 }
 
 
@@ -3319,6 +3357,15 @@ emit_icon_changed (WnckWindow *window)
                  0);
 }
 
+static void
+emit_class_changed (WnckWindow *window)
+{
+  window->priv->need_emit_class_changed = FALSE;
+  g_signal_emit (G_OBJECT (window),
+                 signals[CLASS_CHANGED],
+                 0);
+}
+
 static void
 emit_actions_changed   (WnckWindow       *window,
                         WnckWindowActions changed_mask,
@@ -3336,3 +3383,12 @@ emit_geometry_changed (WnckWindow *window)
                  signals[GEOMETRY_CHANGED],
                  0);
 }
+
+static void
+emit_role_changed (WnckWindow *window)
+{
+  window->priv->need_emit_role_changed = FALSE;
+  g_signal_emit (G_OBJECT (window),
+                 signals[ROLE_CHANGED],
+                 0);
+}
